#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif

const int BLEND_NORMAL = 0;
const int BLEND_SCREEN = 1;
const int BLEND_OVERLAY = 2;
const int BLEND_HARDLIGHT = 3;
const int BLEND_MULTIPLY = 4;
const int BLEND_COLOR_BURN = 5;
const int BLEND_COLOR_DODGE = 6;
const int BLEND_REFLECT = 7;
const int BLEND_GLOW = 8;
const int BLEND_LIGHTEN = 9;
const int BLEND_DARKEN = 10;
///////////////////////////////////////////////////////////
// Uniforms
uniform sampler2D u_texture;
uniform sampler2D u_bgTexture;
uniform int u_blendMode;

///////////////////////////////////////////////////////////
// Varyings
varying vec2 v_texCoord;
varying vec2 v_bgTexCoord;

float blendOverlay(float base, float blend) {
    return base<0.5?(2.0*base*blend):(1.0-2.0*(1.0-base)*(1.0-blend));
}

vec3 blendOverlay(vec3 base, vec3 blend) {
    return vec3(blendOverlay(base.r,blend.r),blendOverlay(base.g,blend.g),blendOverlay(base.b,blend.b));
}

vec3 blendOverlay(vec3 base, vec3 blend, float opacity) {
    return (blendOverlay(base, blend) * opacity + blend * (1.0 - opacity));
}

float blendScreen(float base, float blend) {
    return 1.0-((1.0-base)*(1.0-blend));
}

vec3 blendScreen(vec3 base, vec3 blend) {
    return vec3(blendScreen(base.r,blend.r),blendScreen(base.g,blend.g),blendScreen(base.b,blend.b));
}

vec3 blendScreen(vec3 base, vec3 blend, float opacity) {
    return (blendScreen(base, blend) * opacity + blend * (1.0 - opacity));
}

vec3 blendHardLight(vec3 base, vec3 blend) {
    return blendOverlay(blend,base);
}

vec3 blendHardLight(vec3 base, vec3 blend, float opacity) {
    return (blendHardLight(base, blend) * opacity + blend * (1.0 - opacity));
}

vec3 blendMultiply(vec3 base, vec3 blend) {
    return base*blend;
}

vec3 blendMultiply(vec3 base, vec3 blend, float opacity) {
    return (blendMultiply(base, blend) * opacity + blend * (1.0 - opacity));
}

float blendColorDodge(float base, float blend) {
    return (blend==1.0)?blend:min(base/(1.0-blend),1.0);
}

vec3 blendColorDodge(vec3 base, vec3 blend) {
    return vec3(blendColorDodge(base.r,blend.r),blendColorDodge(base.g,blend.g),blendColorDodge(base.b,blend.b));
}

vec3 blendColorDodge(vec3 base, vec3 blend, float opacity) {
    return (blendColorDodge(base, blend) * opacity + blend * (1.0 - opacity));
}

float blendReflect(float base, float blend) {
    return (blend==1.0)?blend:min(base*base/(1.0-blend),1.0);
}

vec3 blendReflect(vec3 base, vec3 blend) {
    return vec3(blendReflect(base.r,blend.r),blendReflect(base.g,blend.g),blendReflect(base.b,blend.b));
}

vec3 blendReflect(vec3 base, vec3 blend, float opacity) {
    return (blendReflect(base, blend) * opacity + blend * (1.0 - opacity));
}

vec3 blendGlow(vec3 base, vec3 blend) {
    return blendReflect(blend,base);
}

vec3 blendGlow(vec3 base, vec3 blend, float opacity) {
    return (blendGlow(base, blend) * opacity + blend * (1.0 - opacity));
}

float blendColorBurn(float base, float blend) {
    return (blend==0.0)?blend:max((1.0-((1.0-base)/blend)),0.0);
}

vec3 blendColorBurn(vec3 base, vec3 blend) {
    return vec3(blendColorBurn(base.r,blend.r),blendColorBurn(base.g,blend.g),blendColorBurn(base.b,blend.b));
}

vec3 blendColorBurn(vec3 base, vec3 blend, float opacity) {
    return (blendColorBurn(base, blend) * opacity + blend * (1.0 - opacity));
}

float blendLighten(float base, float blend) {
    return max(blend,base);
}

vec3 blendLighten(vec3 base, vec3 blend) {
    return vec3(blendLighten(base.r,blend.r),blendLighten(base.g,blend.g),blendLighten(base.b,blend.b));
}

vec3 blendLighten(vec3 base, vec3 blend, float opacity) {
    return (blendLighten(base, blend) * opacity + blend * (1.0 - opacity));
}

float blendDarken(float base, float blend) {
    return min(blend,base);
}

vec3 blendDarken(vec3 base, vec3 blend) {
    return vec3(blendDarken(base.r,blend.r),blendDarken(base.g,blend.g),blendDarken(base.b,blend.b));
}

vec3 blendDarken(vec3 base, vec3 blend, float opacity) {
    return (blendDarken(base, blend) * opacity + blend * (1.0 - opacity));
}

vec3 execute(vec3 base, vec3 blend, float opacity) {
    if (u_blendMode == BLEND_SCREEN) {
        return blendScreen( base, blend,opacity );
    } else if (u_blendMode == BLEND_OVERLAY){
        return blendOverlay( base, blend ,opacity);
    } else if (u_blendMode == BLEND_HARDLIGHT) {
        return blendHardLight( base, blend ,opacity);
    } else if (u_blendMode == BLEND_MULTIPLY) {
        return blendMultiply( base, blend ,opacity);
    } else if (u_blendMode == BLEND_COLOR_BURN) {
        return blendColorBurn( base, blend ,opacity);
    } else if (u_blendMode == BLEND_COLOR_DODGE) {
        return blendColorDodge( base, blend ,opacity);
    } else if (u_blendMode == BLEND_REFLECT) {
       return blendReflect( base, blend ,opacity);
    } else if (u_blendMode == BLEND_GLOW) {
       return blendGlow( base, blend ,opacity);
    } else if (u_blendMode == BLEND_LIGHTEN) {
       return blendLighten( base, blend ,opacity);
    } else if (u_blendMode == BLEND_DARKEN) {
       return blendDarken( base, blend ,opacity);
    }
    return blend;
}

void main()
{
    if (u_blendMode == BLEND_NORMAL) {
        gl_FragColor = texture2D(u_texture, v_texCoord);
        return;
    }
    vec4 fgColor = texture2D(u_texture, v_texCoord);
    vec4 bgColor = texture2D(u_bgTexture, v_bgTexCoord);
    vec3 color = execute(bgColor.rgb, clamp(fgColor.rgb * (1.0 / fgColor.a), 0.0, 1.0), 1.0);
    gl_FragColor = vec4(bgColor.rgb * (1.0 - fgColor.a) + color.rgb * fgColor.a, 1.0);
}