#define MAXPOINTS 20

attribute vec2 a_texCoord;
varying vec2 v_texCoord;

uniform float aspectRatio;

uniform vec2 center[MAXPOINTS];
uniform vec2 radius[MAXPOINTS];
uniform float scale[MAXPOINTS];
uniform float angle[MAXPOINTS];
uniform float u_min[MAXPOINTS];
uniform float u_max[MAXPOINTS];
uniform int types[MAXPOINTS];

uniform int count;
uniform float progress;

void main()
{
    vec2 uv = a_texCoord;
    gl_Position = vec4(uv * 2.0 - 1.0, 0.0, 1.0);
    for (int i = 0; i < count; i++)
    {
    
        if(scale[i] == 0.0 || types[i] == 0) {
            continue;
        }
        
        vec2 textureCoordinateToUse = uv;
        
        float e1 =  ( textureCoordinateToUse.x - center[i].x ) / ( radius[i].x );
        float e2 =  ( textureCoordinateToUse.y - center[i].y ) / ( radius[i].y / aspectRatio );
        float d  = (e1 * e1) + (e2 * e2);
        
   
        if (d < 1.0)
        {
            if(types[i] == 1) {
                // bulge
                vec2 dist = vec2(d * radius[i].x, d * radius[i].y);
                textureCoordinateToUse -= center[i];
                
                vec2 delta = ((radius[i] - dist) / radius[i]);
                float deltaScale = scale[i];
                if(deltaScale > 0.0) {
                    deltaScale = smoothstep(u_min[i], u_max[i], deltaScale);
                }
                
                vec2 percent = 1.0 - ((delta * deltaScale) * progress);
                textureCoordinateToUse = textureCoordinateToUse * percent;
                uv = textureCoordinateToUse + center[i];
                
            } else if(types[i] == 2) {
                // shift
                float dist = 1.0 - d;
                float delta = scale[i] * dist * progress;

                float deltaScale = smoothstep(u_min[i], u_max[i], dist);
                float directionX = cos(angle[i]) * deltaScale;
                float directionY = sin(angle[i]) * deltaScale / (3.0/4.0 * aspectRatio);
                
                uv = vec2(textureCoordinateToUse.x - (delta * directionX), textureCoordinateToUse.y - (delta * directionY));
            }
        }
    }
    v_texCoord = uv;
}