const highp  vec4  kRGBToYPrime = vec4 (0.299, 0.587, 0.114, 0.0);
const highp  vec4  kRGBToI     = vec4 (0.595716, -0.274453, -0.321263, 0.0);
const highp  vec4  kRGBToQ     = vec4 (0.211456, -0.522591, 0.31135, 0.0);

const highp  vec4  kYIQToR   = vec4 (1.0, 0.9563, 0.6210, 0.0);
const highp  vec4  kYIQToG   = vec4 (1.0, -0.2721, -0.6474, 0.0);
const highp  vec4  kYIQToB   = vec4 (1.0, -1.1070, 1.7046, 0.0);

precision highp float;

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform bool drawFace;

highp float lum(lowp vec3 c) {
    return dot(c, vec3(0.3, 0.59, 0.11));
}

lowp vec3 clipcolor(lowp vec3 c) {
    highp float l = lum(c);
    lowp float n = min(min(c.r, c.g), c.b);
    lowp float x = max(max(c.r, c.g), c.b);

    if (n < 0.0) {
        c.r = l + ((c.r - l) * l) / (l - n);
        c.g = l + ((c.g - l) * l) / (l - n);
        c.b = l + ((c.b - l) * l) / (l - n);
    }
    if (x > 1.0) {
        c.r = l + ((c.r - l) * (1.0 - l)) / (x - l);
        c.g = l + ((c.g - l) * (1.0 - l)) / (x - l);
        c.b = l + ((c.b - l) * (1.0 - l)) / (x - l);
    }

    return c;
}

lowp vec3 setlum(lowp vec3 c, highp float l) {
    highp float d = l - lum(c);
    c = c + vec3(d);
    return clipcolor(c);
}

highp float sat(lowp vec3 c) {
    lowp float n = min(min(c.r, c.g), c.b);
    lowp float x = max(max(c.r, c.g), c.b);
    return x - n;
}

lowp float mid(lowp float cmin, lowp float cmid, lowp float cmax, highp float s) {
    return ((cmid - cmin) * s) / (cmax - cmin);
}

lowp vec3 setsat(lowp vec3 c, highp float s) {
    if (c.r > c.g) {
        if (c.r > c.b) {
            if (c.g > c.b) {
                /* g is mid, b is min */
                c.g = mid(c.b, c.g, c.r, s);
                c.b = 0.0;
            } else {
                /* b is mid, g is min */
                c.b = mid(c.g, c.b, c.r, s);
                c.g = 0.0;
            }
            c.r = s;
        } else {
            /* b is max, r is mid, g is min */
            c.r = mid(c.g, c.r, c.b, s);
            c.b = s;
            c.r = 0.0;
        }
    } else if (c.r > c.b) {
        /* g is max, r is mid, b is min */
        c.r = mid(c.b, c.r, c.g, s);
        c.g = s;
        c.b = 0.0;
    } else if (c.g > c.b) {
        /* g is max, b is mid, r is min */
        c.b = mid(c.r, c.b, c.g, s);
        c.g = s;
        c.r = 0.0;
    } else if (c.b > c.g) {
        /* b is max, g is mid, r is min */
        c.g = mid(c.r, c.g, c.b, s);
        c.b = s;
        c.r = 0.0;
    } else {
        c = vec3(0.0);
    }
    return c;
}

highp float getHue(highp vec4 color)
{
    highp float   YPrime  = dot (color, kRGBToYPrime);
    highp float   I      = dot (color, kRGBToI);
    highp float   Q      = dot (color, kRGBToQ);

    // Calculate the hue and chroma
    highp float   hue     = atan (Q, I);

    return hue;
}


highp vec4 replaceY(highp vec4 src, highp vec4 dest)
{
    highp float   YPrimeDest  = dot (dest, kRGBToYPrime);
    highp float   YPrime  = dot (src, kRGBToYPrime);
    highp float   I      = dot (src, kRGBToI);
    highp float   Q      = dot (src, kRGBToQ);

    highp vec4 color = src;

    highp vec4    yIQ   = vec4 (YPrimeDest, I, Q, 0.0);
    color.r = dot (yIQ, kYIQToR);
    color.g = dot (yIQ, kYIQToG);
    color.b = dot (yIQ, kYIQToB);

    return color;
}

highp vec4 replaceHue(highp vec4 src, highp vec4 dest)
{
    highp vec4 color = src;
    highp float   YPrime  = dot (src, kRGBToYPrime);
    highp float   I      = dot (src, kRGBToI);
    highp float   Q      = dot (src, kRGBToQ);

    // Calculate the hue and chroma
    highp float   hue     = atan (Q, I);
    highp float   chroma  = sqrt (I * I + Q * Q);

    hue = (getHue(dest) + hue)/2.0;
    // Make the user's adjustments
    //hue += (-hueAdjust); //why negative rotation?
    // Convert back to YIQ
    Q = chroma * sin (hue);
    I = chroma * cos (hue);

    // Convert back to RGB
    highp vec4    yIQ   = vec4 (YPrime, I, Q, 0.0);
    color.r = dot (yIQ, kYIQToR);
    color.g = dot (yIQ, kYIQToG);
    color.b = dot (yIQ, kYIQToB);
    return color;
}


highp vec4 replaceH(highp vec4 src, highp float h)
{
    highp vec4 color = src;
    highp float   YPrime  = dot (src, kRGBToYPrime);
    highp float   I      = dot (src, kRGBToI);
    highp float   Q      = dot (src, kRGBToQ);

    // Calculate the hue and chroma
    highp float   hue     = atan (Q, I);
    highp float   chroma  = sqrt (I * I + Q * Q);

    if( abs(hue-h) > 0.4) {
        return color;
    }

    h = (hue+ h)*0.5;

    if(h <= 0.0) {
        return color;
    }

    // Make the user's adjustments
    //hue += (-hueAdjust); //why negative rotation?
    // Convert back to YIQ
    Q = chroma * sin (h);
    I = chroma * cos (h);

    // Convert back to RGB
    highp vec4    yIQ   = vec4 (YPrime, I, Q, 0.0);
    color.r = dot (yIQ, kYIQToR);
    color.g = dot (yIQ, kYIQToG);
    color.b = dot (yIQ, kYIQToB);
    return color;
}

highp vec4 replaceLum(highp vec4 src, highp vec4 dest)
{
    highp float lum = lum(dest.rgb);
    return vec4(setlum(src.rgb, lum), src.a);
}

highp vec4 replaceSat(highp vec4 src, highp vec4 dest)
{
    highp float sat = sat(dest.rgb);
    return vec4(setsat(src.rgb, sat), src.a);
}

void main()
{
    lowp vec4 base = texture2D(inputImageTexture, textureCoordinate);
    highp vec4 color = texture2D(inputImageTexture2, textureCoordinate2);
    highp float hue = getHue(base);
    highp vec4 result;
    if(color.a == 0.0) {
        result = base;
    } else {
        result = replaceH(vec4(color.rgb/color.a, 1.0), hue);
    }
    gl_FragColor = vec4( (result.rgb*color.a + base.rgb*(1.0 - color.a))*1.0 , 1.0);
}